/*
 * MemoryFile.cpp
 *
 *  Created on: Mar 31, 2014
 *      Author: omergilad
 */

#include "memoryfile.h"

namespace Common {

MemoryFile::MemoryFile(const uint8* data, uint32 length) :
		mData(data), mSize(length), mPos(0) {
}

MemoryFile::~MemoryFile() {
}

bool MemoryFile::err() const {
	return false;
}

void MemoryFile::clearErr() {
}

bool MemoryFile::eos() const {
	return (mPos >= mSize);
}

int32 MemoryFile::pos() const {
	return mPos;
}

int32 MemoryFile::size() const {
	return mSize;
}

bool MemoryFile::seek(int32 offs, int whence) {
	switch (whence) {
	case SEEK_END:

		mPos = mSize + offs;
		break;
	case SEEK_SET:
		mPos = offs;
		break;

	case SEEK_CUR:
		mPos += offs;
		break;
	}

	assert(mPos <= mSize);

	return true;
}

uint32 MemoryFile::read(void* dataPtr, uint32 dataSize) {

	// Read at most as many bytes as are still available...
	if (dataSize > mSize - mPos) {
		dataSize = mSize - mPos;
	}

	memcpy(dataPtr, (void*)(mData + mPos), dataSize);

	mPos += dataSize;

	return dataSize;
}

}

/* namespace Common */
